//######################################################################################
//#                                                                                    #
//# Author       : Sadek A. Nehmeh                                                     #
//# Institution  : Memorial Sloan-Kettering Cancer Center                              #
//# Email        : nehmehs@mskcc.org                                                   # 
//#                                                                                    #
//# Program      : Gate_To_Stir.c                                          # 
//# Version      : V.1.0 (12/13/2004)                                                  #
//#                                                                                    #   
//# Objective    : Read MichelogramF.ima From GATE, and writes in STIR format.         #
//#                                                                                    #
//#                                                                                    # 
//# Input        : MichelogramF.ima                                                    #
//#                                                                                    #
//#                                                                                    #
//#                                                                                    #
//# Output       : projections.stir  (Input to projections.hs)                         #
//#                Format: "F" => float (4 Bytes).                                     #
//#                                                                                    #
//#                                                                                    #
//######################################################################################

//P.S. Many of the declared variables are not used, and need to be cleaned up.

{
  gROOT->Reset();
  FILE  *fileMichelogramF;
  FILE  *fileMichelogramSTIR;
  FILE  *fileProjectionsSTIR;
  
  int nRings = 18;                // Number of rings in the GE Advance scanner
  int Num_sinos = nRings*nRings;       
  int   nPHI = 336;               // Number of angular bins in the sinogram   
  int   nU = 283;                 // Number of radial bins in the sinogram
  int nV;                         // Number of projection planes
  int Num_Proj_planes;
  float tmp0[Num_sinos*nPHI*nU], sinogram_STIR_out[Num_sinos][nPHI][nU];
  
  int i, j, k;
  int  ring1, ring2, r1, r2, segment_num, entry, u, phi, v;
  float tmp_array[nPHI][nU],sinogram[nRings][nRings][nPHI][nU], tmp1[Num_sinos*nPHI*nU];
  short int ans;
  int num_segments;
  int num_slices_per_segment[11] =  {35,16,15,14,13,12,11,10,9,8,7};  //Number of projection planes per segment

  fileMichelogramF     = fopen("MichelogramF.ima","rb");
  fileMichelogramSTIR  = fopen("MichelogramF.stir","wb");
  fileProjectionsSTIR  = fopen("projections.stir","wb");


  //Read the Michelogram data...
  //----------------------------
  ans =  fread(tmp1,4,(Num_sinos*nPHI*nU) , fileMichelogramF);
  entry = 0;
  
  printf("Please enter the absoute number of segments you would like to reconstruct (0, -> 10):");
  scanf("%d", &num_segments);

  Num_Proj_planes = num_slices_per_segment[0];
  for (i=1; i<(num_segments+1); i++)    
    Num_Proj_planes += 2*num_slices_per_segment[i];
  
  printf("Generating projections.stir; please wait...\n");  

  //Read Michelogram into "sinogram" ...   
  for (ring1=0; ring1<nRings; ring1++)
    {
      for (ring2=0; ring2<nRings; ring2++)
	{	            
	  for (phi=0; phi<nPHI; phi++)
	    {
	      for (u=0; u<nU; u++)	       
		{	    		  
		  sinogram[ring1][ring2][phi][u] = tmp1[entry];		
		  entry +=1;
		}
	    }
	}
    }

  
  
  //Process segments 11->2
  //-----------------------
  printf("Processing segments 11 -> 2... \n");
  for (ring1= (num_segments+1); ring1>1; ring1--)
    {     
      int c = 0;
      r1 = ring1;
      while (r1<17)
	{
	  r1 = ring1 +  c;
	  r2 = r1 - ring1;
	  segment_num = ring1; 
	  ans =  fwrite(sinogram[r1][r2],4,(nPHI*nU) , fileMichelogramSTIR);
	  c += 1;
	}
      
      
    }

  

  
  //process Segment 0 (delta-ring= -1,0,+1)
  //---------------------------------------
  printf("Processing segment zero... \n");
  for (ring1=0; ring1<nRings; ring1++)
    {
      r1 = ring1;
      ans =  fwrite(sinogram[r1][r1],4,(nPHI*nU) , fileMichelogramSTIR);      
      
      if (r1 < (nRings-1))
	{
	  for (phi=0; phi<nPHI; phi++)
	    {
	      for (u=0; u<nU; u++)
		{
		  tmp_array[phi][u] = sinogram[r1+1][r1][phi][u] + sinogram[r1][r1+1][phi][u];	  
		}
	    }
	  ans =  fwrite(tmp_array,4,(nPHI*nU) , fileMichelogramSTIR);
	}
    }
  
  //                         ****************************  //
  
  //goto NEXT2;
  
  //Process segments -2->-11
  //------------------------
  printf("Processing segments 2 -> 11... \n");
  for (ring2=2; ring2< (num_segments + 2); ring2++)
    {      
      int c = 0;
      r2 = ring2;
      segment_num = (-1)*ring2;
      while (r2 < (nRings-1 ) )
	{
	  r2 = ring2 +  c;
	  r1 =  r2 -  ring2;
	  //printf("segment_num = %d     :    r1 = %d      ------   r2 = %d\n", segment_num,r1,r2);
	  ans =  fwrite(sinogram[r1][r2],4,(nPHI*nU) , fileMichelogramSTIR);
	  c += 1;
	}
      
    }
  
  
  fclose(fileMichelogramF);
  fclose(fileMichelogramSTIR);
 
  
  
  //                       ################################       //
  

  //Now, re-open the MichelogramF.stir, and generate the projections.stir file....
  //------------------------------------------------------------------------------
  
  fileMichelogramSTIR  = fopen("MichelogramF.stir","rb");
        
  
  for (segment_num = num_segments ; segment_num >= ( (-1)*num_segments); segment_num--)    
    {               
      nV= num_slices_per_segment[abs(segment_num)];
      printf("Number of Slices in Segment [%d] is: %d\n",segment_num, num_slices_per_segment[abs(segment_num)]);
     

      //Declare some dynamic arrays
      //---------------------------
      
      float* tmp2;
      tmp2 = new float [nV*nPHI*nU];
     
     
      float*** sinogram_STIR;           
      sinogram_STIR = new float **[nV];
      for (v=0; v<nV;v++)
	{
	  sinogram_STIR[v] = new float *[nPHI];
	  for (phi=0; phi<nPHI; phi++)
	    sinogram_STIR[v][phi] = new float [nU];      
	}

     
      float*** Projections ;
      Projections   = new float **[nPHI];
       for (phi=0; phi < nPHI; phi++)
	{
	  Projections[phi] = new float *[nV];
	  for (v=0; v < nV; v++)
	    Projections[phi][v] = new float [nU];      
	}


      
       tmp_array2   = new float **[nV];
       for (v=0; v < nV; v++)	 
	 tmp_array2[v] = new float *[nU];
	  

       //--------------------------------
 
       printf("Reading MichelogramFstir...\n");
       ans =  fread(tmp2,4, (num_slices_per_segment[abs(segment_num)])*nPHI*nU , fileMichelogramSTIR); //Read the first segment from fileMichelogramSTIR
       
       printf("Reading the Sinogram data...\n");
       entry =0 ;       
       for (v=0; v < num_slices_per_segment[abs(segment_num)]; v++) 
	 {
	   for (phi=0; phi<nPHI; phi++)
	     {	       	 		   
	       for (u=0; u<nU; u++)	       
		 {
		   sinogram_STIR[v][phi][u] =  tmp2[entry];
		  
		   entry +=1;		   
		 }
	     }    	 	  
	 }   

   
     
      
       printf("Arraging the sinogram data into projection planes...\n");           
       for (phi=0; phi<nPHI; phi++)
	 {
	   for (v=0; v < num_slices_per_segment[abs(segment_num)]; v++) 
	     {  
	       for (u=0; u<nU; u++)	       
		 {	    		  
		   Projections[phi][v][u] = sinogram_STIR[v][phi][u];
		 }
	     }    
	 }       
       
       
       for (phi=0; phi<nPHI; phi++)
	 {
	   for (v=0; v < num_slices_per_segment[abs(segment_num)]; v++) 
	     {  
	       for (u=0; u<nU; u++)	       
		 {	    		  		   
		   tmp_array[v][u] = sinogram_STIR[v][phi][u];
		 }
	     }
	   ans =  fwrite(tmp_array ,4,(nU*num_slices_per_segment[abs(segment_num)]) , fileProjectionsSTIR);
	 }
       
       
      

       //Delete the arrays...
       //--------------------
       delete [] Projections;
       delete [] sinogram_STIR;

       //--------------------
      
    }
  
  fclose (fileMichelogramSTIR);
  fclose (fileProjectionsSTIR);
  
}     
      
